package com.seitenbau.testing.shared.config;

import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.LinkedList;
import java.util.List;

import com.seitenbau.testing.shared.helper.ReflectionHelper;
import com.seitenbau.testing.shared.tracer.ITracer;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * Methoden Konfiguration
 *
 * Beschreibt die zu berwachende/aufzuzeichnende Methode
 */
public class MethodConfiguration
{

  private String PackageName = "";

  private String ClassName = "Klasse";

  private String MethodName = "Methode";

  private String ReturnType = "void";

  private String Parameters = "";

  private boolean enableRecordPreState = false;

  private boolean enableRecordPostState = false;

  /** bei statischen methoden wird immer der static Part gespeichert. */
  private boolean enableRecordStaticPart = false;

  private boolean enableRecordReturnValue = true;

  private boolean enableRecordParameterValues = true;

  private boolean enableRecordTrace = false;

  private TracerConfiguration Tracer = null;

  @XStreamImplicit(itemFieldName = "ExcludeField")
  private List<ExcludeFieldConfiguration> ExcludeFields = null;

  public MethodConfiguration()
  {

  }

  public MethodConfiguration(String classPackage, String className,
      String returnType, String methodName, String parameterTypes)
  {
    setPackageName(classPackage);
    setClassName(className);
    setReturnType(returnType);
    setMethodName(methodName);
    setParameters(parameterTypes);
  }

  /**
   * Convenient method to build up the Structure directly from a Class
   *
   * <p>It is <b>NOT</b> recommend to use this method in a productive environment.
   * It will just use the first found method, matching the given methodName !!!!
   *
   * @param clazzWithMethod
   * @param methodName
   */
  public MethodConfiguration(Class<?> clazzWithMethod, String methodName)
  {
    setupByClass(clazzWithMethod, methodName);
  }

  public String getPackageName()
  {
    return PackageName;
  }

  public void setPackageName(String classPackage)
  {
    this.PackageName = classPackage;
  }

  public String getClassName()
  {
    return ClassName;
  }

  public void setClassName(String className)
  {
    this.ClassName = className;
  }

  public String getMethodName()
  {
    return MethodName;
  }

  public void setMethodName(String methodName)
  {
    this.MethodName = methodName;
  }

  public String getReturnType()
  {
    if (ReturnType == null)
    {
      return "*";
    }
    if (ReturnType.equals(""))
    {
      return "*";
    }
    return ReturnType;
  }

  public void setReturnType(String returnType)
  {
    this.ReturnType = returnType;
  }

  public String getParameters()
  {
    if (Parameters == null)
    {
      return "";
    }
    return Parameters;
  }

  public void setParameters(String parameterTypes)
  {
    this.Parameters = parameterTypes;
  }

  public TracerConfiguration getTracer()
  {
    if (Tracer == null)
    {
      Tracer = new TracerConfiguration();
    }
    return Tracer;
  }

  /**
   * Convenient method to set the Tracer.
   *
   * <p>
   * Implicitly generates a new TracerConfiguration which is used to
   * wrap the passed Class Object.
   *
   * @param tracerClazz The Class of the Tracer you really want to
   *        have.
   */
  public void setTracer(Class<? extends ITracer> tracerClazz)
  {
    setTracer(new TracerConfiguration(tracerClazz));
  }

  public void setTracer(TracerConfiguration aTracer)
  {
    Tracer = aTracer;
  }

  public List<ExcludeFieldConfiguration> getExcludeFields()
  {
    if (ExcludeFields == null)
    {
      ExcludeFields = new LinkedList<ExcludeFieldConfiguration>();
    }
    return ExcludeFields;
  }

  public void setExcludeFields(List<ExcludeFieldConfiguration> excludedFields)
  {
    ExcludeFields = excludedFields;
  }

  public void addExcludeFields(ExcludeFieldConfiguration excludedField)
  {
    getExcludeFields().add(excludedField);
  }

  public boolean isEnableRecordPreState()
  {
    return enableRecordPreState;
  }

  public void setEnableRecordPreState(boolean enableRecordPreState)
  {
    this.enableRecordPreState = enableRecordPreState;
  }

  public boolean isEnableRecordPostState()
  {
    return enableRecordPostState;
  }

  public void setEnableRecordPostState(boolean enableRecordPostState)
  {
    this.enableRecordPostState = enableRecordPostState;
  }

  public boolean isEnableRecordReturnValue()
  {
    return enableRecordReturnValue;
  }

  public void setEnableRecordReturnValue(boolean enableRecordReturnValue)
  {
    this.enableRecordReturnValue = enableRecordReturnValue;
  }

  public boolean isEnableRecordParameterValues()
  {
    return enableRecordParameterValues;
  }

  public void setEnableRecordParameterValues(boolean enableRecordParameterValues)
  {
    this.enableRecordParameterValues = enableRecordParameterValues;
  }

  public boolean isEnableRecordTrace()
  {
    return enableRecordTrace;
  }

  public void setEnableRecordTrace(boolean enableRecordTrace)
  {
    this.enableRecordTrace = enableRecordTrace;
  }

  public boolean isEnableRecordStaticPart()
  {
    return enableRecordStaticPart;
  }

  public void setEnableRecordStaticPart(boolean enableRecordStaticPart)
  {
    this.enableRecordStaticPart = enableRecordStaticPart;
  }

  protected void setupByClass(Class<?> clazzWithMethod, String methodName)
  {
    Method method = ReflectionHelper.getFirstMatchingMethod(clazzWithMethod, methodName);
    if (method != null)
    {
      setupByClass(clazzWithMethod, methodName, method);
    }
    else
    {
      System.err.println("Unable to find Method");
    }
  }

  public static MethodConfiguration buildFor(Class<?> clazz, String methodName)
  {
    Method method = ReflectionHelper.getFirstMatchingMethod(clazz, methodName);
    if (method != null)
    {
      System.out.print("buildFor : ");
      System.out.println(method);
      MethodConfiguration mcfg = new MethodConfiguration();

      mcfg.setupByClass(clazz, methodName, method);

      return mcfg;
    }
    else
    {
      throw new IllegalArgumentException("Unable to find Method!" + methodName
          + " in class " + clazz);
    }
  }

  protected void setupByClass(Class<?> clazz, String methodName,
      Method method)
  {
    setMethodName(methodName);
    setClassName(clazz.getSimpleName());
    setPackageName(clazz.getPackage().getName());

    Class<?> returnType = method.getReturnType();
    setReturnType(returnType.getCanonicalName());

    Type[] types = method.getGenericParameterTypes();
    StringBuffer params = new StringBuffer();
    for (int i = 0; i < types.length; ++i)
    {
      Type tt = types[i];
      if(tt instanceof Class<?>)
      {
        params.append( ((Class<?>)tt).getCanonicalName() );
      }
      else
      {
      params.append(tt);
      }
      if (i < types.length - 1)
      {
        params.append(", ");
      }
    }

    setParameters(params.toString());
  }

}
