package com.seitenbau.testing.shared.helper;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import sun.misc.Unsafe;

/**
 * Provides convenient methods for using reflection.
 */
public final class ReflectionHelper
{
  /** Only static Methods */
  private ReflectionHelper()
  {
  }

  public static Object createInstance(String className)
      throws InstantiationException, IllegalAccessException,
      ClassNotFoundException
  {
    Class<?> clazz = Class.forName(className);
    return clazz.newInstance();
  }

  /**
   * Versucht eine Instanz zum bergebenen Klassennamen zu erzeugen.
   *
   * @param clazzName Der Klassenname in FQN Form. Beispielsweise
   *        "java.lang.String"
   *
   * @return Die konkrete Instanz oder null wenn die Klasse nicht
   *         instanzierbar war.
   */
  public static Object createInstanceSilent(String clazzName)
  {
    Object obj = null;
    try
    {
      obj = createInstance(clazzName);
    }
    catch (Exception e)
    {
      return null;
    }
    return obj;
  }

  public static Object createInstanceSilent(ClassLoader cl, String clazzName)
  {
    Object obj = null;
    try
    {
      Class<?> clazz = cl.loadClass(clazzName);
      obj = clazz.newInstance();
    }
    catch (Exception e)
    {
      return null;
    }
    return obj;
  }

  /** Die Unsafe Instanz */
  static Unsafe myUnsafeInstance = getUnsafe();

  /**
   * Unsafe Instanz holen.
   *
   * @return Die Unsafe Instanz
   */
  protected static Unsafe getUnsafe()
  {
    try
    {
      Field field = Unsafe.class.getDeclaredField("theUnsafe");
      field.setAccessible(true);
      return (Unsafe) field.get(null);
    }
    catch (Exception ex)
    {
      throw new RuntimeException("can't get Unsafe instance", ex);
    }
  }

  /**
   * Wirft die angegebene Exception, throws muss nicht angegeben sein.
   *
   * Mit dieser Methode kann eine Exception geworfen werden obwohl
   * kein throws an der Methode angegeben ist. Diese Methode sollte
   * nur genutzt werden wenn unbedingt ntig.
   *
   * Da die Methode eine Exception wirft, entspricht einem throw new
   * xxx() Allerdings versteht dies der Compiler nicht, wesswegen aus
   * sicht des Compilers der Code an dieser nicht verlassen wird.
   * Daher kann ein return null; ntig sein um Compiler Fehler zu
   * unterdrcken. whrend der Laufzeit hat dies allerdings keine
   * Auswirkung, da das Werfen der Exception ganz normal die aktuelle
   * Methode 'unterbricht'.
   *
   * @param throwable Die zu werfende Exception.
   *
   */
  public static void throwException(Throwable throwable)
  {
    myUnsafeInstance.throwException(throwable);
  }

  public static <TYPE> TYPE createInstanceSilent(Class<? extends TYPE> clazz)
  {
    Object obj = null;
    try
    {
      obj = clazz.newInstance();
    }
    catch (Exception e)
    {
      return null;
    }
    return Generics.cast(obj);
  }

  public static Object getFieldValueContaining(Object instance, Class<?> clazz)
  {
    if (instance == null || clazz == null)
    {
      return null;
    }
    Field[] fields = instance.getClass().getDeclaredFields();
    for (Field field : fields)
    {
      try
      {
        field.setAccessible(true);
        Object value = field.get(instance);
        if (clazz.isInstance(value))
        {
          return value;
        }
      }
      catch (IllegalArgumentException e)
      {
        e.printStackTrace();
      }
      catch (IllegalAccessException e)
      {
        e.printStackTrace();
      }
    }
    return null;
  }

  public static Method getFirstMatchingMethod(Class<?> clazz, String methodName)
  {
    Method[] methods = clazz.getDeclaredMethods();
    for (Method method : methods)
    {
      if (method.getName().equals(methodName))
      {
        return method;
      }
    }
    return null;
  }

  public static boolean isPrimitiveArray(Class<?> suspectedClass)
  {
    if (suspectedClass.isArray())
    {
      while (suspectedClass.isArray())
        suspectedClass = suspectedClass.getComponentType();
      return suspectedClass.isPrimitive();
    }
    return false;
  }

  public static Method getMethod(Class<?> clazz, String methodName,
      String[] parameterTypes)
  {
    Class<?>[] classTypes = new Class<?>[parameterTypes.length];
    for (int i = 0; i < parameterTypes.length; i++)
    {
      String type = parameterTypes[i];
      Class<?> klasse;
      try
      {
        klasse = Class.forName(type);
      }
      catch (ClassNotFoundException e)
      {
        throw new RuntimeException(e);
      }
      classTypes[i] = klasse;
    }

    return getMethod(clazz, methodName, classTypes);
  }

  public static Method getMethod(Class<?> clazz, String methodName,
      Class<?>[] classTypes)
  {
    try
    {
      return clazz.getDeclaredMethod(methodName, classTypes);
    }
    catch (Exception e)
    {
      throw new RuntimeException(e);
    }
  }

}
