package com.seitenbau.testing.shared.model;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.List;

import org.custommonkey.xmlunit.XMLAssert;
import org.xml.sax.SAXException;

import com.seitenbau.testing.shared.config.ExcludeFieldConfiguration;
import com.seitenbau.testing.shared.xstream.HideControlCharXppDriver;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.ConversionException;

public class ObjectSnapshotConverter
{
  private static final int AJC_LEN = 3;
  private XStream myXStream = null;

  public ObjectSnapshotConverter(List<ExcludeFieldConfiguration> fields)
  {
    myXStream = new XStream(new HideControlCharXppDriver());
    myXStream.omitField(Throwable.class, "stackTrace");
    myXStream.autodetectAnnotations(true);

    if (fields != null)
    {
      for (ExcludeFieldConfiguration efCfg : fields)
      {
        try
        {
          myXStream.omitField(Class.forName(efCfg.getClassName()), efCfg
              .getFieldName());
          System.out.println("XStream omits: " + efCfg.getClassName() + "::"
              + efCfg.getFieldName());
        }
        catch (ClassNotFoundException e)
        {
          System.out.println("Unable to omit: " + efCfg.getClassName() + "::"
              + efCfg.getFieldName());
        }
      }
    }
  }

  private XStream getXStream()
  {
    if (myXStream == null)
    {
      myXStream = new XStream();
    }
    return myXStream;
  }

  public Object makeObject(ObjectSnapshot objectSnapshot)
  {
    if (objectSnapshot == null)
    {
      throw new IllegalArgumentException(
          "ObjectSnapshot: Parameter objectSnapshot was null");
    }
    return getXStream().fromXML(objectSnapshot.getXMLString());
  }

  public ObjectSnapshot makeSnapshot(Object target, boolean recordStaticPart)
  {
    ObjectSnapshot os = makeSnapshot(target);
    if (recordStaticPart && target != null)
    {
      HashMap<String, Object> staticPart = getStaticMap(target.getClass());
      os.setStaticXMLString(getXStream().toXML(staticPart));
    }
    return os;
  }

  public ObjectSnapshot makeSnapshotStatic(Class<?> clazz)
  {
    ObjectSnapshot os = new ObjectSnapshot();

    HashMap<String, Object> staticPart = getStaticMap(clazz);
    os.setStaticXMLString(getXStream().toXML(staticPart));

    return os;
  }

  public ObjectSnapshot makeSnapshot(Object obj)
  {
    ObjectSnapshot os = new ObjectSnapshot();
    XStream xs = getXStream();

    // If you get a Exception a la :
    //
    // com.thoughtworks.xstream.converters.ConversionException: Could
    // not call java.util.concurrent.ConcurrentHashMap.writeObject() :
    // Cannot handle CGLIB enhanced proxies with multiple callbacks
    // ---- Debugging information ----
    // -------------------------------
    // message : Could not call
    // java.util.concurrent.ConcurrentHashMap.writeObject()
    // cause-exception :
    // com.thoughtworks.xstream.converters.ConversionException
    // cause-message : Cannot handle CGLIB enhanced proxies with
    // multiple callbacks
    // -------------------------------
    //
    // Then probably an EntityManager is inside the Object to
    // serialize. But this is not possible.
    // Try to omit that field. (e.g. @XStreamOmitField)

    try
    {
      os.setXMLString(xs.toXML(obj));
    }
    catch (ConversionException ce)
    {
      System.err
          .println("If you use Hibernate / JTA, try add a @XStreamOmitField to the EntityManger field!");
      throw ce;
    }

    os.setMetaInfo(xs.toXML(new ObjectSnapshotMetaInfo(obj)));

    return os;
  }

  private HashMap<String, Object> getStaticMap(Class<?> targetClass)
  {
    HashMap<String, Object> map = new HashMap<String, Object>();
    for (Field field : targetClass.getDeclaredFields())
    {
      String name = field.getName();
      // Ignore AspectJ thisJointPoints
      // XXX: Unsaubere Lsung fr ajc Felder:
      if (name.length() <= AJC_LEN || !name.substring(0, 3).equals("ajc"))
      {
        field.setAccessible(true);
        if (Modifier.isStatic(field.getModifiers()))
        {
          try
          {
            map.put(name, field.get(null));
          }
          catch (Exception e)
          {
            e.printStackTrace();
          }
        }
      }
    }
    return map;
  }

  // public ObjectSnapshot makeSnapshot(Object obj,boolean
  // writeMetaInfo) {
  // if(writeMetaInfo) {
  // return makeSnapshot(obj);
  // } else {
  // ObjectSnapshot os = new ObjectSnapshot();
  // os.setXMLString( getXStream().toXML(obj) );
  // return os;
  // }
  // }

  public void assertEquals(Object obj, Object compObj)
  {
    try
    {
      String xml1 = getXStream().toXML(obj);
      String xml2 = getXStream().toXML(compObj);

      XMLAssert.assertXMLEqual(xml1, xml2);
    }
    catch (SAXException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

}
