package com.seitenbau.testing.shared.tracer;


import org.aspectj.lang.JoinPoint;

import com.seitenbau.testing.shared.model.RecordedTest;

/**
 * Basis Implementinerung eines Tracer zum Aufbau des
 * Aufzeichnungs-Modelles
 */
public abstract class AbstractTracer extends AbstractTracerWithListener
    implements IRequireDao
{
  /**
   * @see ITracer#targetMethodEntering(JoinPoint)
   */
  public void targetMethodEntering(JoinPoint thisJoinPoint)
  {
    // this is THE method. So start tracing
    try
    {
      // Pre initialize any AspectJ Aware Listener.
      fireAspectJAwareEntering(thisJoinPoint);


      RecordedTest newRecordedTestInstance=createRecordedTest();
      setRecordedTestInstance( newRecordedTestInstance );

      savePreState(newRecordedTestInstance, thisJoinPoint);

      savePreStateValues(newRecordedTestInstance, thisJoinPoint);


      // Static Target Method?
      if (thisJoinPoint.getTarget() == null)
      { // Yes is static
        // Following code returns the class name, the method is
        // implemented in
        newRecordedTestInstance.setClassName(thisJoinPoint.getSignature()
            .getDeclaringTypeName());
      }
      else
      {
        // Following code returns the class name the method (in
        // memory)
        // is of (RTTI)
        newRecordedTestInstance.setClassName(thisJoinPoint.getTarget()
            .getClass().getName());
      }

      newRecordedTestInstance.setMethodName(thisJoinPoint.getSignature()
          .getName());
      int mod = thisJoinPoint.getSignature().getModifiers();
      newRecordedTestInstance.setModifiers(mod);

      fireEnteringMethod();
    }
    catch (Throwable throwed)
    {
      try
      {
        handleInternalExceptionTargetEntering(throwed, thisJoinPoint);
      }
      catch (Throwable throwed2)
      {
        System.err.println(
                "targetMethodEntering() : internal Exception catched and removed."
                + throwed2
            );
      }
    }
  }

  protected void handleInternalExceptionTargetEntering(Throwable throwed,
      JoinPoint thisJoinPoint)
  {
    System.err.println("AbstractTracer::handleInternalExceptionTargetEntering"
        + throwed);
  }

  protected void fireAspectJAwareEntering(JoinPoint thisJoinPoint)
  {
    for (ITracerListener listener : getListeners())
    {
      if (listener instanceof IAspectJAware)
      {
        ((IAspectJAware) listener).enteringTarget(thisJoinPoint);
      }
    }
  }

  protected void fireAspectJAwareThrowedException(JoinPoint thisJoinPoint,
      Throwable theThrowable)
  {
    for (ITracerListener listener : getListeners())
    {
      if (listener instanceof IAspectJAware)
      {
        ((IAspectJAware) listener).exceptionThrownTarget(thisJoinPoint,
            theThrowable);
      }
    }
  }

  protected void fireAspectJAwareProceeded(JoinPoint thisJoinPoint)
  {
    for (ITracerListener listener : getListeners())
    {
      if (listener instanceof IAspectJAware)
      {
        ((IAspectJAware) listener).proceededTarget(thisJoinPoint);
      }
    }
  }

  /**
   * @see ITracer#targetMethodProceeded(JoinPoint, Object)
   */
  public void targetMethodProceeded(JoinPoint thisJoinPoint,
      Object theReturnValue)
  {
    try
    {
      fireAspectJAwareProceeded(thisJoinPoint);
      savePostStateValues(getRecordedTestInstance(), thisJoinPoint, theReturnValue);

      fireLeavingMethod(theReturnValue);

      finishUp();
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Error! =====");
      throwed.printStackTrace();
    }
  }

  /**
   * @see ITracer#subMethodEntering(JoinPoint)
   */
  public void subMethodEntering(JoinPoint thisJoinPoint)
  {
    try
    {
      if (getRecordedTestInstance() != null
          && getConfigurationMethod().isEnableRecordTrace())
      {
        callToMethodDao = getDaoFactory().getCallToMethodDao();

        mySubMethodInstance = callToMethodDao.createNew(getRecordedTestInstance());

        savePreState(mySubMethodInstance, thisJoinPoint);
        savePreStateValues(mySubMethodInstance, thisJoinPoint);

        if (thisJoinPoint.getTarget() == null)
        {
          // Following code would only return the class name the
          // method
          // is implemented in
          mySubMethodInstance.setClassName(thisJoinPoint.getSignature()
              .getDeclaringTypeName());
        }
        else
        {
          // Following code would return the class name the method in
          // memory is of (RTTI)
          mySubMethodInstance.setClassName(thisJoinPoint.getTarget().getClass()
              .getName());
        }

        mySubMethodInstance.setMethodName(thisJoinPoint.getSignature()
            .getName());
        mySubMethodInstance.setModifiers(thisJoinPoint.getSignature()
            .getModifiers());

        fireSubMethodEntering();
      }
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Error! =====");
      throwed.printStackTrace();
    }

  }

  /**
   * @see ITracer#subMethodProceeded(JoinPoint, Object)
   */
  public void subMethodProceeded(JoinPoint thisJoinPoint, Object theReturnValue)
  {
    try
    {
      if (mySubMethodInstance != null
          && getConfigurationMethod().isEnableRecordTrace())
      {
        if (getConfigurationMethod().isEnableRecordTrace())
        {
          savePostStateValues(mySubMethodInstance, thisJoinPoint,
              theReturnValue);
        }
        fireSubMethodLeaving( theReturnValue);

        finishMethodCall();
      }
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Error! =====");
      throwed.printStackTrace();
    }
  }

  /**
   * @see ITracer#subNewOperatorEntering(JoinPoint)
   */
  public void subNewOperatorEntering(JoinPoint thisJoinPoint)
  {
    try
    {
      if (myRecordedTestInstance != null
          && getConfigurationMethod().isEnableRecordTrace())
      {
        newOerationDao = getDaoFactory().getCallToBaseNewOperationDao();

        myNewOperatorInstance = newOerationDao.create(myRecordedTestInstance);

        savePreState(myNewOperatorInstance, thisJoinPoint);
        savePreStateValues(myNewOperatorInstance, thisJoinPoint);

        myNewOperatorInstance.setClassName(thisJoinPoint.getStaticPart()
            .getSignature().getDeclaringTypeName());

        fireNewOperatorEntering();
      }
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Error! =====");
      throwed.printStackTrace();
    }
  }

  /**
   * @see ITracer#subNewOperatorProceeded(JoinPoint, Object)
   */
  public void subNewOperatorProceeded(JoinPoint thisJoinPoint,
      Object theReturnValue)
  {
    try
    {
      if (myNewOperatorInstance != null
          && getConfigurationMethod().isEnableRecordTrace())
      {
        if (getConfigurationMethod().isEnableRecordTrace())
        {
          savePostStateValues(myNewOperatorInstance, thisJoinPoint,
              theReturnValue);
        }
        fireNewOperatorLeaving(theReturnValue);
        finishNewOperator();

      }
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Error! =====");
      throwed.printStackTrace();
    }
  }

  public void subMethodThrowedException(JoinPoint thisJoinPoint,
      Throwable theThrowable)
  {
    try
    {
      if (mySubMethodInstance != null
          && getConfigurationMethod().isEnableRecordTrace())
      {
        setThrowable(mySubMethodInstance, theThrowable);
        fireSubMethodThrowedException(theThrowable);
        finishMethodCall();
      }
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Error! =====");
      throwed.printStackTrace();
    }
  }

  public void subNewOperatorThrowedException(JoinPoint thisJoinPoint,
      Throwable t)
  {
    try
    {
      if (myNewOperatorInstance != null
          && getConfigurationMethod().isEnableRecordTrace())
      {
        setThrowable(myNewOperatorInstance, t);
        fireNewOperatorThrowedException( t);
        finishNewOperator();
      }
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Error! =====");
      throwed.printStackTrace();
    }
  }

  public void targetMethodThrowedException(JoinPoint thisJoinPoint,
      Throwable theThrowable)
  {
    try
    {
      fireAspectJAwareThrowedException(thisJoinPoint, theThrowable);
      if (myRecordedTestInstance != null)
      {
        setThrowable(myRecordedTestInstance, theThrowable);
        fireThrowedException( theThrowable);
        finishUp();
      }
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Error! =====");
      throwed.printStackTrace();
    }
  }

}
