package com.seitenbau.testing.shared.tracer;

import java.util.List;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.reflect.CodeSignature;
import org.aspectj.lang.reflect.ConstructorSignature;
import org.aspectj.lang.reflect.MethodSignature;

import com.seitenbau.testing.shared.config.ExcludeFieldConfiguration;
import com.seitenbau.testing.shared.config.MethodConfiguration;
import com.seitenbau.testing.shared.dao.IDaoFactory;
import com.seitenbau.testing.shared.dao.IInvocationMethodDao;
import com.seitenbau.testing.shared.dao.IInvocationNewOperatorDao;
import com.seitenbau.testing.shared.model.InvocationBase;
import com.seitenbau.testing.shared.model.InvocationMethod;
import com.seitenbau.testing.shared.model.InvocationNewOperator;
import com.seitenbau.testing.shared.model.ObjectSnapshot;
import com.seitenbau.testing.shared.model.ObjectSnapshotConverter;
import com.seitenbau.testing.shared.model.RecordedTest;

public abstract class AbstractTracerWithHelpers implements ITracer
{

  protected IInvocationNewOperatorDao newOerationDao;

  protected IInvocationMethodDao callToMethodDao;

  protected InvocationNewOperator myNewOperatorInstance;

  protected InvocationMethod mySubMethodInstance;

  protected RecordedTest myRecordedTestInstance;

  protected IDaoFactory myDaoFactory;

  protected MethodConfiguration myConfigurationMethod = null;

  protected ObjectSnapshotConverter myObjectSnapshotConverter;

  protected List<ExcludeFieldConfiguration> myCurrentExcludeFields;

  /**
   * When Enterning a Method, a RecordedTest is needed. Do it.
   *
   * This is abstract, so you can return your own subclass or connect
   * the RecordedTest into the Model/Database.
   *
   * @return The RecordedTest Instance
   */
  protected abstract RecordedTest createRecordedTest();

  /**
   * If you need, now you should write the model to the Database.
   *
   * @param theRecordedTest The RecordedTest you can/should write now
   *        into the Database or not. The Tracer will 'forget' about
   *        this instance directly after this method.
   */
  protected abstract void saveRecordedTest(RecordedTest theRecordedTest);

  /**
   * Get the Method Configuration.
   *
   * @return the Method Configuration.
   */
  public MethodConfiguration getConfigurationMethod()
  {
    return myConfigurationMethod;
  }

  /**
   * Set the Method Configuration.
   *
   * @param configurationMethod The Method Configuration.
   */
  public void setConfigurationMethod(MethodConfiguration configurationMethod)
  {
    myConfigurationMethod = configurationMethod;
    setCurrentExcludeFields(myConfigurationMethod.getExcludeFields());
  }

  private void setupXStream(List<ExcludeFieldConfiguration> excludedFields)
  {
    if (excludedFields == null)
    {
      return;
    }
//    setCurrentExcludeFields(excludedFields);
    myObjectSnapshotConverter = new ObjectSnapshotConverter(excludedFields);
  }

  protected void finishUp()
  {
    saveRecordedTest(myRecordedTestInstance);
    myRecordedTestInstance = null;
  }

  protected void finishMethodCall()
  {
    mySubMethodInstance = null;
  }

  protected void finishNewOperator()
  {
    myNewOperatorInstance = null;
  }

  /**
   * Get the current DaoFactory
   *
   * @return
   */
  public IDaoFactory getDaoFactory()
  {
    return myDaoFactory;
  }

  /**
   * @see IRequireDao#setDaoFactory(IDaoFactory)
   */
  public void setDaoFactory(IDaoFactory daoFactory)
  {
    myDaoFactory = daoFactory;
  }

  /**
   * Speichern des PostStatus
   *
   * @param theMethodItem Die Methoden Instanz fr welche der Status
   *        gespeichert werden soll.
   *
   * @param thisJoinPoint Der JointPoint
   *
   * @param returnValue Der Rckgabewert der Methode
   *
   */
  protected void savePostStateValues(InvocationBase theMethodItem,
      JoinPoint thisJoinPoint, Object returnValue)
  {

      if (myConfigurationMethod.isEnableRecordPostState())
      {
        theMethodItem.setPostState(getObjectState(thisJoinPoint));
      }
      if (theMethodItem instanceof InvocationNewOperator)
      {
        theMethodItem.setReturnValue(makeSnapshot(returnValue));
      }
      if (myConfigurationMethod.isEnableRecordReturnValue())
      {
        if (theMethodItem instanceof InvocationMethod
            && !((InvocationMethod) theMethodItem).isVoidMethod())
        {
          theMethodItem.setReturnValue(makeSnapshot(returnValue));
        }
      }

  }

  /**
   * Erzeugt ber die DaoFactory einen ObjectSnapshot.
   *
   * @param obj Objekt von welchen der SnapShot gemacht werden soll.
   *
   * @return Der gemacht SnapShot
   *
   */
  private ObjectSnapshot makeSnapshot(Object obj)
  {
      if (getConfigurationMethod().isEnableRecordStaticPart())
      {
        return getObjectSnapshotConverter().makeSnapshot(obj, true);
      }
      else
      {
        return getObjectSnapshotConverter().makeSnapshot(obj);
      }

  }

  /**
   * Liefert den Status des Objektes.
   *
   * @param thisJoinPoint
   *
   * @return Der Status als SnapShot
   */
  private ObjectSnapshot getObjectState(JoinPoint thisJoinPoint)
  {
    Object target = thisJoinPoint.getTarget();
    if (target == null)
    { // Is a Static Method !
      Class<?> clazz = thisJoinPoint.getSignature().getDeclaringType();
      return getObjectSnapshotConverter().makeSnapshotStatic(clazz);
    }

    return makeSnapshot(target);
  }

  /**
   * Speichert den Pre Status
   *
   * @param theMethodItem Das modell in welchem der PreState gesetzt
   *        werden soll
   *
   * @param thisJoinPoint
   *
   */
  protected void savePreState(InvocationBase theMethodItem,
      JoinPoint thisJoinPoint)
  {

      CodeSignature cs = (CodeSignature) thisJoinPoint.getSignature();

      if (cs instanceof MethodSignature)
      {
        MethodSignature ms = (MethodSignature) cs;
        theMethodItem.setReturnType(ms.getReturnType());
      }
      else if (cs instanceof ConstructorSignature)
      {
        ConstructorSignature conSig = (ConstructorSignature) cs;
        theMethodItem.setReturnType(conSig.getDeclaringTypeName());
      }

      theMethodItem.setArgumentTypes(cs.getParameterTypes());

  }

  protected void savePreStateValues(InvocationBase theMethodItem,
      JoinPoint thisJoinPoint)
  {

      if (myConfigurationMethod.isEnableRecordPreState())
      {
        theMethodItem.setPreState(getObjectState(thisJoinPoint));

      }
      if (myConfigurationMethod.isEnableRecordParameterValues())
      {
        if (thisJoinPoint.getArgs().length != 0)
        {
          Object[] args = thisJoinPoint.getArgs();
          ObjectSnapshot[] osArgs = new ObjectSnapshot[args.length];
          int i = 0;
          for (Object arg : args)
          {
            try {
            osArgs[i] = makeSnapshot(arg);
            } catch(RuntimeException t) {
              System.err.println("Error while serializeing - "
                  +"JoinPoint: " + thisJoinPoint
                  +"Argument["+i+"] : " + arg
                  );
              throw t;
            }
            i++;
          }
          theMethodItem.setArgumentValues(osArgs);
        }
      }

  }

  /**
   * Liefert die erzeugte RecordedTest Instanz zurck
   *
   * @return Die erzeugte RecordedTest Instanz.
   */
  public RecordedTest getRecordedTest()
  {
    return myRecordedTestInstance;
  }

  protected void setThrowable(InvocationBase callTo, Throwable t)
  {
    if (callTo != null)
    {
      callTo.setExceptionThrown(true);
      String name = t.getClass().getName();
      callTo.setExceptionType(name);
      callTo.setExceptionValue(makeSnapshot(t));
    }
  }

  public ObjectSnapshotConverter getObjectSnapshotConverter()
  {
    if (myObjectSnapshotConverter == null)
    {
      myObjectSnapshotConverter = new ObjectSnapshotConverter(null);
    }
    return myObjectSnapshotConverter;
  }

  public void setObjectSnapshotConverter(
      ObjectSnapshotConverter objectSnapshotConverter)
  {
    myObjectSnapshotConverter = objectSnapshotConverter;
  }

  public List<ExcludeFieldConfiguration> getCurrentExcludeFields()
  {
    return myCurrentExcludeFields;
  }

  public void setCurrentExcludeFields(
      List<ExcludeFieldConfiguration> currentExcludeFields)
  {
    if(currentExcludeFields!=null)
    {
      setupXStream( currentExcludeFields );
    }
    myCurrentExcludeFields = currentExcludeFields;
  }

  public InvocationMethod getSubMethodInstance()
  {
    return mySubMethodInstance;
  }

  public void setSubMethodInstance(InvocationMethod subMethodInstance)
  {
    mySubMethodInstance = subMethodInstance;
  }

  public InvocationNewOperator getNewOperatorInstance()
  {
    return myNewOperatorInstance;
  }

  public void setNewOperatorInstance(InvocationNewOperator newOperatorInstance)
  {
    myNewOperatorInstance = newOperatorInstance;
  }

  public IInvocationMethodDao getCallToMethodDao()
  {
    return callToMethodDao;
  }

  public void setCallToMethodDao(IInvocationMethodDao callToMethodDao)
  {
    this.callToMethodDao = callToMethodDao;
  }

  public IInvocationNewOperatorDao getNewOerationDao()
  {
    return newOerationDao;
  }

  public void setNewOerationDao(IInvocationNewOperatorDao newOerationDao)
  {
    this.newOerationDao = newOerationDao;
  }

  public RecordedTest getRecordedTestInstance()
  {
    return myRecordedTestInstance;
  }

  public void setRecordedTestInstance(RecordedTest recordedTestInstance)
  {
    myRecordedTestInstance = recordedTestInstance;
  }

}
