package com.seitenbau.testing.shared.tracer;


import org.aspectj.lang.JoinPoint;

import com.seitenbau.testing.shared.config.MethodConfiguration;

/**
 * Tracer werden per Komposition in den Aspekt eingebunden
 * 
 * Ansatt das Mitschreiben von Methodenaufrufe im Aspekt selbst zu
 * implementieren wird dies ber eine Komposition durch einem Tracer
 * erledigt.
 */
public interface ITracer
{

  /**
   * Eintritt in die Ziel Methode
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   */
  void targetMethodEntering(JoinPoint thisJoinPoint);

  /**
   * Nach dem Aufruf der Zielmethode
   * 
   * Die Zielmethode wurde aufgerufen.
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   * @param theReturnValue Der Rckgabewert der Ziel Methode. null bei
   *        void methoden.
   * 
   */
  void targetMethodProceeded(JoinPoint thisJoinPoint, Object theReturnValue);

  /**
   * Eintritt in einen Methodenaufruf innerhalb der Zielmethode
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   */
  void subMethodEntering(JoinPoint thisJoinPoint);

  /**
   * Austritt aus Aufruf einer unter Methode innerhalb der Zielmethode
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   * @param theReturnValue Der Rckgabewert der Methode. null bei void
   *        methoden.
   */
  void subMethodProceeded(JoinPoint thisJoinPoint, Object theReturnValue);

  /**
   * Beginn eines new Operator aufrufes innerhalb der Zielmethode
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   */
  void subNewOperatorEntering(JoinPoint thisJoinPoint);

  /**
   * Ende eines new Operator Aufrufes innerhalb der Zielmethode
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   * @param theReturnValue Der Rckgabewert der Methode. null bei void
   *        methoden.
   */
  void subNewOperatorProceeded(JoinPoint thisJoinPoint, Object theReturnValue);

  /**
   * Setzt die Methoden Konfiguration.
   * 
   * Der Tracer kann hierber weitere Einstellungen aus der Methoden
   * Konfiguration herrauslesen.
   * 
   * @param mcfg Die Methoden Konfiguration.
   * 
   */
  void setConfigurationMethod(MethodConfiguration mcfg);

  /**
   * 
   * Die Zielmethode wirft eine Exception
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   * @param aThrowable Die Exception, genauer eine Throwable, da diese
   *        die Wurzel der Java-Exceptions ist.
   */
  void targetMethodThrowedException(JoinPoint thisJoinPoint,
      Throwable aThrowable);

  /**
   * 
   * Eine Unter-Methode hat eine Exception geworfen
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   * @param aThrowable Die geworfene Exception, genauer eine
   *        Throwable, da diese die Wurzel der Java-Exceptions ist.
   */
  void subMethodThrowedException(JoinPoint thisJoinPoint, Throwable aThrowable);

  /**
   * 
   * Ein new() aufruf hat eine Exception geworfen.
   * 
   * @param thisJoinPoint Der zugehrige AspectJ JointPoint
   * 
   * @param aThrowable Die geworfene Exception, genauer eine
   *        Throwable, da diese die Wurzel der Java-Exceptions ist.
   */
  void subNewOperatorThrowedException(JoinPoint thisJoinPoint,
      Throwable aThrowable);

}
