package com.seitenbau.testing.shared.tracer;


import com.seitenbau.testing.shared.config.MethodConfiguration;
import com.seitenbau.testing.shared.config.RecorderConfiguration;
import com.seitenbau.testing.shared.dao.IRecordedTestCaseDao;
import com.seitenbau.testing.shared.dao.IRecordedTestDao;
import com.seitenbau.testing.shared.helper.LoggerHelper;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;
import com.thoughtworks.xstream.XStream;

/**
 * Implementierung des Tracer zum Mitschneiden whrend der
 * Programmausfhrung.
 */
public class RecordTracer extends AbstractTracer
{

  /**
   * Zugehriger RecordedTestCase, pro Lauf eine Insanz.
   */
  private RecordedTestCase myRecordedTestCase = null;

  /** Die Methoden Konfiguration */
  private RecordedMethodConfiguration myRecordConfig;

  /**
   * Default Konstruktor ntig, da die TracerFactory Reflections zur
   * Instanzierung nutzt.
   */
  public RecordTracer()
  {
    LoggerHelper.logger.info("RecordTracer::<init>");
  }

  /**
   * Fr Jeden Test-Fall ist ein neuer Recorded Test ntig.
   *
   * <p>
   * Dieser wird hier neu angelegt, Dabei wird dieser an den aktuellen
   * RecordedTestCase angehngt.
   */
  protected RecordedTest createRecordedTest()
  {
    RecordedTestCase testCase = getRecordedTestCase();

    RecordedTest recordedTest = createNewRecordedTestFor(testCase);

    return recordedTest;
  }

  /**
   * Rezeugt einen neuen RecordedTest fr einen RecordedTestCase
   *
   * @param recordedTestCase
   * @return Der neue RecordedTestCase, welcher in den
   *         RecordedTestCase eingeklinkt ist.
   *
   */
  private RecordedTest createNewRecordedTestFor(
      RecordedTestCase recordedTestCase)
  {
    IRecordedTestDao dao = getDaoFactory().getRecordedTestDao();
    RecordedTest recordedTest = dao.createNew(recordedTestCase);

    return recordedTest;
  }

  /**
   * Liefert den akutellen RecordedTestCase, oder erzeugt einen neuen.
   * @return
   */
  protected RecordedTestCase getRecordedTestCase()
  {
    if (myRecordedTestCase == null)
    {
      myRecordedTestCase = createNewRecordedTestCase();
    }
    return myRecordedTestCase;
  }

  /**
   * Erzeugt einen neuen RecordedTestCase und speichert die Konfiguration in das
   * Datenbank feld ab
   *
   * @return
   */
  protected RecordedTestCase createNewRecordedTestCase()
  {
    IRecordedTestCaseDao dao = getDaoFactory().getRecordedTestCaseDao();

    RecordedTestCase recordedTestCase = dao.createNew(getRecordConfig());

    recordedTestCase.setName( getNewRecordedTestCaseName() );

    XStream xs = RecorderConfiguration.getXStream();

    String xml = xs.toXML(getConfigurationMethod());

    recordedTestCase.setConfigurationSnapshot(xml);

    fireInitTestCase(recordedTestCase);

    return recordedTestCase;
  }

  /**
   * Liefert den gewnschten Namen fr einen neuen RecordedTestCase benutzt wird.
   * @return
   */
  protected String getNewRecordedTestCaseName()
  {
    return "default";
  }

  /**
   * RecordedMethodConfiguration aus der Datenbank lesen oder neu
   * anlegen.
   *
   * @return Liefert das aktuelle 'wurzel'-Element, die
   *         RecordedMethodConfiguration aus der Datenbank. Existiert
   *         keine wird dieses angelegt.
   */
  protected RecordedMethodConfiguration getRecordConfig()
  {
    if (myRecordConfig == null)
    {
      MethodConfiguration mcfg = getConfigurationMethod();
      myRecordConfig = getDaoFactory().getRecordConfigDao().findOrCreateBy(
          mcfg.getPackageName(), mcfg.getClassName(), mcfg.getMethodName(),
          mcfg.getReturnType(), mcfg.getParameters());
    }
    return myRecordConfig;
  }

  /**
   * Abspeichern des aufgezeichneten Tests in die Datenbank.
   */
  protected void saveRecordedTest(RecordedTest theRecordedTest)
  {
    IRecordedTestCaseDao dao = getDaoFactory().getRecordedTestCaseDao();

    dao.save(myRecordedTestCase);
  }

}
