package com.seitenbau.testing.shared.tracer;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.HashMap;

import org.aspectj.lang.JoinPoint;

import com.seitenbau.testing.shared.helper.LoggerHelper;
import com.seitenbau.testing.shared.helper.ReflectionHelper;
import com.seitenbau.testing.shared.rmi.ClassTree;
import com.seitenbau.testing.shared.rmi.ClassTreeImpl;
import com.seitenbau.testing.shared.rmi.ClassTreeWithChild;
import com.seitenbau.testing.shared.rmi.ClassTreeWithChildImpl;
import com.seitenbau.testing.shared.rmi.IRecorderController;

public class RmiTracer extends RecordTracer
{

  static
  {
    // Just to get 1 log Message hopfully at Application startup, to
    // see if we can break
    // out of 'the box' via RMI.
    IRecorderController controller = null;
    try
    {
      controller = (IRecorderController) Naming
          .lookup(IRecorderController.RMI_ID);
      controller.logMessage("RmiTracer::static<init>");
    }
    catch (Throwable anything)
    {
    }
    finally
    {
      controller = null;
    }
  }

  private IRecorderController myRmiController;

  protected IRecorderController getRmiController()
  {
    if (myRmiController == null)
    {
      myRmiController = connectRmiController();
    }
    return myRmiController;
  }

  protected IRecorderController connectRmiController()
  {
    IRecorderController controller = null;
    try
    {
      controller = (IRecorderController) Naming
          .lookup(IRecorderController.RMI_ID);
      controller.ping();
      controller.registerRecorder(null);

    }
    catch (Throwable anything)
    {
      LoggerHelper.logger.warning(anything.toString());
      controller = null;
    }
    return controller;
  }

  @Override
  protected void handleInternalExceptionTargetEntering(Throwable throwed,
      JoinPoint thisJoinPoint)
  {
    IRecorderController ctrl = getRmiController();
    if (ctrl != null)
    {
      logMessage(ctrl, "JoinPoint : " + thisJoinPoint.toString());
      logMessage(ctrl, "Throwable : " + throwed.toString());
      logMessage(ctrl, "handleInternalExceptionTargetEntering");

      Object target = thisJoinPoint.getTarget();

      HashMap<Object, ClassTree> ignore = new HashMap<Object, ClassTree>();

      ClassTreeWithChildImpl root = new ClassTreeWithChildImpl();
      if(target!=null) {
        root.setNode(target.getClass().getCanonicalName());
        root.setNodeName(target.toString());
      } else {
        root.setNodeName( thisJoinPoint.getSignature().getDeclaringTypeName() );
      }

      root.addChild(buildTree(ignore, target, "state",20));
//      logMessage(ctrl,"Array count :" + thisJoinPoint.getArgs().length);
//      for(Object arg:thisJoinPoint.getArgs()) {
//        logMessage(ctrl," > " + arg.toString() );
//      }
      root.addChild( buildTree2(ignore, thisJoinPoint.getArgs(), 20 ) );
      if (root != null)
      {
        sendObjectTree(ctrl, root);
      }
    }
  }

  private ClassTree buildTree2(HashMap<Object, ClassTree> ignore,
      Object target, int depth)
  {
    if (target.getClass().isArray()
        && !ReflectionHelper.isPrimitiveArray(target.getClass()))
    {
      return buildTree2Array( ignore,target,depth );
    } else {
      ClassTreeImpl item = new ClassTreeImpl();
      item.setNode( target.getClass().getCanonicalName() );
      String fieldcount="fields= "+target.getClass().getDeclaredFields().length;
      item.setNodeName( fieldcount );
      return item;
    }
  }

  private ClassTree buildTree2Array(HashMap<Object, ClassTree> ignore,
      Object target, int depth)
  {
    if (target.getClass().isArray()
        && !ReflectionHelper.isPrimitiveArray(target.getClass()))
    {
      ClassTreeWithChildImpl childs = new ClassTreeWithChildImpl();
      childs.setNode( target.getClass().getCanonicalName());
      Object[] array = (Object[]) target;
      int i=0;
      for(Object item:array) {
        ClassTree sub = buildTree2(ignore, item, depth-1);
        if(sub instanceof ClassTreeWithChild) {
          sub.setNodeName(" idx = " + i);
        }
        i++;
        childs.addChild( sub );
      }

      return childs;
    } else {
      return null;
    }
  }

  private void sendObjectTree(IRecorderController ctrl,
      ClassTreeWithChildImpl root)
  {
    try
    {
      ctrl.sendObjectTree(root);
    }
    catch (RemoteException e)
    {
    }
  }

  protected void logMessage(IRecorderController ctrl, String message)
  {
    if(ctrl==null) {
      System.out.println("no rmi controller to log message to : " + message);
    }
    try
    {
      ctrl.logMessage(message);
    }
    catch (RemoteException e)
    {
    }
  }

  private ClassTree buildTree(HashMap<Object, ClassTree> ignore, Object target,
      String name,int currentDepth)
  {
    if (target == null)
    {
      return null;
    }

    if(currentDepth==0) {
      ClassTreeImpl impl = new ClassTreeImpl();
      impl.setNode( "stopping tree excport, depth == 0");
      impl.setNodeName( "MAX_DEPTH" );
      return impl;
    }

    Class<? extends Object> clazz = target.getClass();

    ClassTreeWithChildImpl childs = new ClassTreeWithChildImpl();
    childs.setNode(target.getClass().getCanonicalName());
    childs.setNodeName(name);

    buildArray(ignore, target, childs,currentDepth);

    Field[] fields = clazz.getDeclaredFields();

    for (Field field : fields)
    {
      if (Modifier.isStatic(field.getModifiers()))
      {
        // hmm handle static stuff correct ...
      }
      else
      {
        try
        {
          Object newTarget;
          field.setAccessible(true);
          newTarget = field.get(target);

          if (!ignore.containsKey(newTarget))
          {
            ignore.put(newTarget, null);
            ClassTree child = buildTree(ignore, newTarget, field.getName(),currentDepth-1);
            childs.addChild(child);
          }
          else
          {
            ClassTreeImpl impl = new ClassTreeImpl();
            if (newTarget != null)
            {
              impl.setNode(newTarget.getClass().getCanonicalName());
            }
            else
            {
              impl.setNode(field.getClass().getCanonicalName());
            }
            impl.setNodeName(field.getName());
            childs.addChild(impl);
          }
        }
        catch (Throwable e)
        {
        }
      }
    }
    if (childs.getChilds().isEmpty())
    {
      ClassTreeImpl one = new ClassTreeImpl();
      one.setNode( childs.getNode());
      one.setNodeName( childs.getNodeName());
      return one;
    }
    return childs;
  }

  private void buildArray(HashMap<Object, ClassTree> ignore, Object target,
      ClassTreeWithChildImpl childs,int currentDepth)
  {
    if (target.getClass().isArray()
        && !ReflectionHelper.isPrimitiveArray(target.getClass()))
    {
      Object[] array = (Object[]) target;
       int i = 0;
      for (Object item : array)
      {
        try
        {
          if (!ignore.containsKey(item))
          {
            ignore.put(item, null);
            ClassTree child = buildTree(ignore, item, "[" + i + "]",currentDepth-1);
            childs.addChild(child);
          }
        }
        catch (Throwable t)
        {

        }
        i++;
      }
    }
  }

  public void logMessage(String message)
  {
    logMessage(getRmiController(), message);
  }

  //
  // @Override
  // protected RecordedTest createRecordedTest() {
  // IRecorderController controller = getRMIController();
  // if (controller != null) {
  //
  // // try {
  // // String newName = controller.getRecordedTestCaseName();
  // // if (getRecordedTestCaseName() == null && newName != null) {
  // // createRecordedTestCase(newName);
  // // } else if (controller != null && newName != null) {
  // // if (!getRecordedTestCaseName().equals(newName)) {
  // // createRecordedTestCase(newName);
  // // }
  // // }
  // // } catch (RemoteException e) {
  // // }
  // }
  //
  // return super.createRecordedTest();
  // }
  //
  // protected IRecorderController myRMIController;
  //
  // private void connectToRMI() {
  // if (myRMIController != null) {
  // try {
  // myRMIController.logMessage("called createRecordedTest");
  // } catch (ConnectException ce) {
  // myRMIController=connectToServer();
  // } catch (Exception e) {
  // System.err.println(e);
  // }
  // } else
  // {
  // myRMIController=connectToServer();
  // }
  // }
  //
  // private IRecorderController connectToServer() {
  // IRecorderController controller = null;
  // try {
  // controller = (IRecorderController) Naming
  // .lookup(IRecorderController.RMI_ID);
  // controller.registerRecorder( null );
  // } catch (Exception e) {
  // e.printStackTrace();
  // System.err.println(" Connection lost ..." + e);
  // }
  // return controller;
  // }
  //
  //
  //
  // public RmiTracer() {
  // super();
  // // try
  // // {
  // // myRMIController = (IRecorderController) Naming
  // // .lookup(IRecorderController.RMI_ID);
  // //
  // myRMIController.registerRecorder(RecordTracer.class.getCanonicalName());
  // // myRMIController.registerCallbackHandler( this );
  // // LoggerHelper.logger.info("RMI Controller found & connected :"
  // // + myRMIController);
  // // }
  // // catch (Exception e)
  // // {
  // // LoggerHelper.logger.warning("Unable to find RMI Controller :"
  // + e);
  // // }
  // }
  //
  // public IRecorderController getRMIController() {
  // if (myRMIController == null) {
  // connectToRMI();
  // }
  // return myRMIController;
  // }
  //
  // public void setRMIController(IRecorderController controller) {
  // myRMIController = controller;
  // }
  //
  // public void addExcludeField(String clazzName, String fieldName) {
  // List<ExcludeFieldConfiguration> oldList =
  // getCurrentExcludeFields();
  // if(oldList==null) {
  // oldList=new ArrayList<ExcludeFieldConfiguration>();
  // }
  // ExcludeFieldConfiguration newConfig = new
  // ExcludeFieldConfiguration();
  // newConfig.setClassName(clazzName);
  // newConfig.setFieldName(fieldName);
  // oldList.add(newConfig);
  //
  // setCurrentExcludeFields(oldList);
  // }
  //
  // public String getCurrentTestCaseName() throws RemoteException
  // {
  // return getRecordedTestCase().getName();
  // }
  //
  // // public void finalize() {
  // // System.out.println("final");
  // // if(myRMIController!=null) {
  // // try
  // // {
  // // myRMIController.unregisterRecorder( this );
  // // }
  // // catch (RemoteException e)
  // // {
  // //
  // // e.printStackTrace();
  // // }
  // // myRMIController=null;
  // // }
  // // }

}
