package com.seitenbau.testing.shared.tracer;

import com.seitenbau.testing.shared.config.IListenerConfiguration;
import com.seitenbau.testing.shared.config.MethodConfiguration;
import com.seitenbau.testing.shared.config.TracerConfiguration;
import com.seitenbau.testing.shared.dao.IDaoFactory;
import com.seitenbau.testing.shared.helper.LoggerHelper;
import com.seitenbau.testing.shared.helper.ReflectionHelper;

/**
 * Factory zur Instanzierung der Tracer in Abhnigkiet zur
 * Konfiguration.
 *
 */
public class TracerFactory
{

  /**
   * Instanziert einen durch die Konfiguration spezifizierten Tracer.
   *
   * @param mcfg Methoden Konfiguration. Aus diesem Parameter wird der
   *        Name des zu instanzierenden Tracer ausgelesen.
   *
   * @param daoFactory Wenn der spezifizierte Tracer IRequireDao
   *        implementiert, so wird darber diese DaoFactory gesetzt.
   *
   * @return Den spezifizierten Tracer, oder Falls dies nicht mglich
   *         war, wird der RecorderTracer zurckgeliefert.
   *
   */
  public static ITracer createTracer(MethodConfiguration mcfg,
      IDaoFactory daoFactory)
  {
    TracerConfiguration tracerCfg = mcfg.getTracer();

    ITracer tracer = createTracer(tracerCfg);

    tracer.setConfigurationMethod(mcfg);

    if (tracer instanceof IRequireDao)
    {
      ((IRequireDao) tracer).setDaoFactory(daoFactory);
    }

    doAddTracerListeners(tracer, tracerCfg, daoFactory, mcfg);

    LoggerHelper.logger.info( "Tracer created = " + tracer );

    return tracer;
  }

  /**
   * Instanziert den spezifizeirten Tracer
   *
   * @param recorder Die Konfiguration.
   *
   * @return Den konkreten Tracer. Im Fehlerfall, wird der NullTracer
   *         zurckgeliefert. Falls kein Tracer spezifiziert wurde,
   *         wird der RecordTracer zurckgeleifert.
   */
  private static ITracer createTracer(TracerConfiguration recorder)
  {
//    Verschoben nach AbstractTracer<init>
//    if( ! TracerControl.isTracerEnabled() )
//    {
//      // Tracer sind global deaktiviert!
//      return new NullTracer();
//    }
    if (recorder == null || recorder.getTracerClass() == null
        || recorder.getTracerClass().length() < 1)
    {
      return new RecordTracer();
    }

    Object obj = ReflectionHelper.createInstanceSilent(recorder
        .getTracerClass());
    if (obj == null)
    {
      System.err.println("Was not able to create " + recorder.getTracerClass());
      return new NullTracer();
    }
    if (obj instanceof ITracer)
    {
      return (ITracer) obj;
    }
    else
    {
      System.err.println(recorder.getTracerClass() + " is not implementing "
          + ITracer.class.getCanonicalName());
      return new NullTracer();
    }
  }

  /**
   * Fgt in einen bergebenen Tracer Listener ein sofern dieser ein
   * ITracerWithListener.
   *
   * @param tracer Der Tracer in welchen die Listern eingefgt werden,
   *        sofern dieser ITracerWithListener implementiert.
   *
   * @param recorder Das Tracer Konfigurationsmodell. Hierherraus wird
   *        die Listeners Liste verarbeitet und alle Listener die
   *        ITracerListener implementieren in die Listener Liste des
   *        Tracers hinzugefgt.
   */
  private static void doAddTracerListeners(ITracer tracer,
      TracerConfiguration recorder, IDaoFactory daoFactory,
      MethodConfiguration mcfg)
  {

    if (!(tracer instanceof ITracerWithListener) || recorder == null
        || recorder.getListeners() == null || recorder.getListeners().isEmpty())
    {
      return;// Nothing to do
    }

    ITracerWithListener lTracer = (ITracerWithListener) tracer;

    for (IListenerConfiguration lcfg : recorder.getListeners())
    {
      Class<? extends ITracerListener> clazz = lcfg.getListenerClass();
      if (clazz != null)
      {
        Object obj = ReflectionHelper.createInstanceSilent(clazz);
        // Object
        // obj=ReflectionHelper.createInstanceSilent(lcfg.getClassName());
        if (obj instanceof ITracerListener)
        {
          ITracerListener listener = (ITracerListener) obj;
          if(listener instanceof ITracerAwareListener)
          {
            ((ITracerAwareListener)listener).registerTracer( tracer );
          }
          listener.initListener(mcfg, lcfg, daoFactory);
          lTracer.addTracerListener(listener);
        }
        else
        {
          System.err.println(clazz + " is not implementing ITracerListener.");
        }
      }
    }
  }
}
